import { IGraph, PluginBase, IPluginBaseConfig } from '@antv/g6';
export interface MapViewConfig extends IPluginBaseConfig {
    key?: string;
    /** Container for map's container. */
    container?: HTMLDivElement | null;
    /** Class name of map's container. */
    className?: string;
    /** Size of map view. */
    size?: number[];
    /** Theme of the map, 'auto' means follow the graph. */
    theme?: 'light' | 'dark' | 'auto';
    /** Whether enable the brush select on map. */
    enableBrushSelect?: boolean;
    /** Whether enable the hover to activate nodes/edges on map. */
    enableHoverActivate?: boolean;
    /** Whether enable the click to select nodes/edges on map. */
    enableSelect?: boolean;
    /** The activate state name of graph items. */
    activateState?: string;
    /** The select state name of graph items. */
    selectState?: string;
    /** The key on keyboard as assistance of brush. Drag without the key down means drag map. */
    brushKey?: string;
    /** The initial zoom of the map. */
    initialMapZoom?: number;
    /** The initial view center of the map. Initially focus on China.*/
    initialMapCenter?: [number, number];
    /** The css style of the brush DOM. */
    brushCSS?: Partial<CSSStyleDeclaration>;
}
export declare class MapView extends PluginBase {
    private scene;
    private container;
    private lineLayer;
    private pointLayer;
    private labelLayer;
    private transientLineActiveLayer;
    private transientPointActiveLayer;
    private transientLineSelectLayer;
    private transientPointSelectLayer;
    private inited;
    constructor(options?: MapViewConfig);
    getDefaultCfgs(): MapViewConfig;
    getEvents(): {
        afterrender: () => void;
        afteritemchange: () => void;
        afteritemstatechange: (e: any) => void;
    };
    init(graph: IGraph): void;
    /**
     * Init the DOM container for map.
     */
    initMap(): Promise<unknown>;
    private newPointLayer;
    private newLineLayer;
    private addMapListeners;
    private updateMap;
    private updateMapState;
    private onKeyDown;
    private onKeyUp;
    private onBrushing;
    private onBrushEnd;
    private onBrush;
    private setMapItemState;
    private setItemState;
    destroy(): void;
}
